// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"
#include "assert/assert.hpp"
#include "event/Event.hpp"

namespace Dawn {
  class AssetManager;
  
  class Asset {
    public:
      AssetManager *assetManager;
      std::string name;
      uint8_t state = 0x00;
      bool loaded = false;
      Event<> eventLoaded;

      /**
       * Create an abstract Asset object.
       * 
       * @param assetManager Asset manager that this asset belongs to.
       * @param name Name of the asset.
       */
      Asset(AssetManager *assetManager, std::string name);

      /**
       * Virtual function that will be called by the asset manager on a 
       * synchronous basis. This will only trigger if the blocks are false and
       * the loaded is also false.
       */
      virtual void updateSync() = 0;

      /**
       * Virtual function called by the asset manager asynchronously every tick.
       * This will only trigger if blocks are false and the loaded state is also
       * false.
       */
      virtual void updateAsync() = 0;
      
      /**
       * Dispose the asset item.
       */
      virtual ~Asset();
  };
}