// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TicTacToeGame.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

TicTacToeGame::TicTacToeGame(SceneItem *item) : SceneItemComponent(item) {}

void TicTacToeGame::onStart() {
  this->tiles = getScene()->findComponents<TicTacToeTile>();

  getScene()->eventSceneUpdate.addListener(this, &TicTacToeGame::onSceneUpdate);
}

void TicTacToeGame::onSceneUpdate() {
  // Get mouse in screen space.
  auto mouse = getGame()->inputManager.getAxis2D(INPUT_BIND_MOUSE_X, INPUT_BIND_MOUSE_Y);
  mouse *= 2.0f;
  mouse -= glm::vec2(1, 1);

  Camera *camera = getScene()->findComponent<Camera>();
  if(camera == nullptr) return;

  struct Ray3D ray;
  glm::vec3 pos = glm::vec3(mouse.x * camera->orthoRight, mouse.y * camera->orthoBottom, 0.0f);
  ray.direction = glm::vec3(0, 0, -15);
  ray.origin = pos;

  getScene()->debugRay((struct SceneDebugRay){ .start = ray.origin, .direction = ray.direction });
}